"""
:mod:`tests` package.

Test suites for the :mod:`etlplus` package.

Notes
-----
This package contains unit tests for CLI, extraction, validation,
transformation, and loading functionality.
"""

from __future__ import annotations
