"""
:mod:`tests.smoke` subpackage.

Smoke test suites for the :mod:`etlplus` package.

Notes
-----
This package contains smoke tests for CLI, extraction, validation,
transformation, and loading functionality.
"""

from __future__ import annotations
